from datetime import datetime, date
from typing import Any, Iterable, List, Mapping, Optional, Tuple, Union

KwargsType = Union[
    Mapping[
        Union[str, bytes, int, float],
        Union[str, bytes, int, float, Iterable[Union[str, bytes, int, float]]],
    ],
    str,
    bytes,
    None,
    Tuple[
        Union[str, bytes, int, float],
        Union[str, bytes, int, float, Iterable[Union[str, bytes, int, float]]],
    ],
    Mapping[
        str,
        Union[str, bytes, int, float, Iterable[Union[str, bytes, int, float]]],
    ],
    Mapping[
        bytes,
        Union[str, bytes, int, float, Iterable[Union[str, bytes, int, float]]],
    ],
    Mapping[
        int,
        Union[str, bytes, int, float, Iterable[Union[str, bytes, int, float]]],
    ],
    Mapping[
        float,
        Union[str, bytes, int, float, Iterable[Union[str, bytes, int, float]]],
    ],
]

JSONType = Union[str, int, float, bool, None, Mapping[str, Any], List[Any]]

ListType = List[Union[str, int, float, bool, List[Union[str, int, float, bool]]]]

DictType = Union[List[Mapping[str, Union[str, int, float, bool]]], JSONType]

_ParamTypes = Union[str, int, float, bool, datetime, date]

ParamsType = Optional[
    Mapping[
        str,
        Union[_ParamTypes, Iterable[_ParamTypes]],
    ]
]



from typing import Union

from pandas import DataFrame  # type: ignore[import]

from datatosk import types
from datatosk.sources.source import SourceRead


class SQLRead(SourceRead):
    """Base class to be inherited by all SQL database based SourceRead classes."""

    def __call__(
        self,
        query: str = "",
        params: types.ParamsType = None,
        **kwargs: types.KwargsType,
    ) -> DataFrame:
        """Default all SQLRead() children .read() implementation."""
        return self.to_pandas(query=query, params=params, **kwargs)

    def to_dict(self) -> Union[types.DictType, types.JSONType]:
        raise NotImplementedError

    def to_list(self) -> types.ListType:
        raise NotImplementedError

    def to_pandas(
        self,
        query: str = "",
        params: types.ParamsType = None,
        **kwargs: types.KwargsType,
    ) -> DataFrame:
        raise NotImplementedError



"""
Datatosk's main interface.

Examples:
    MySQL:
    >>> import datatosk
    >>> source = datatosk.mysql(source_name="dope_db")
    >>> source.read("SELECT * FROM dope_table")
       dope_column  other_dope_column
    0            1                  3
    1            2                  4
"""

from . import sources

# pylint: disable=invalid-name
mysql = sources.databases.sql.mysql.MySQLSource
gbq = sources.databases.sql.google_big_query.GBQSource
pickle = sources.files.PickleSource
requests = sources.network.RequestsSource
mongodb = sources.databases.mongodb.MongoSource



import os
from abc import ABCMeta, abstractmethod
from dataclasses import dataclass
from typing import Dict, List, Optional

from pandas import DataFrame  # type: ignore

from datatosk import types
from datatosk.errors import DatatoskEnvValueError


@dataclass
class SourceParam:
    """Used to define each source initialization parameters."""

    env_name: str
    is_required: bool = True
    default: Optional[str] = None

    def __post_init__(self) -> None:
        if not self.is_required and self.default is None:
            raise ValueError(
                f"Not required param {self.env_name} has no default value."
            )


class SourceRead:
    """Base class to be inherited by all SourceRead classes."""

    def to_pandas(self) -> DataFrame:
        """Reads data from a source and returns it as a pandas DataFrame."""
        raise NotImplementedError

    def to_list(self) -> types.ListType:
        """Reads data from a source and returns it as a list."""
        raise NotImplementedError

    def to_dict(self) -> types.DictType:
        """Reads data from a source and returns it as a dict or list of dicts."""
        raise NotImplementedError


class SourceWrite:
    """Base class to be inherited by all SourceWrite classes."""

    def from_pandas(self, data: DataFrame) -> None:
        """Writes data to a source from a pandas DataFrame."""
        raise NotImplementedError

    def from_list(self, data: types.ListType) -> None:
        """Writes data to a source from a list."""
        raise NotImplementedError

    def from_dict(self, data: types.DictType) -> None:
        """Writes data to a source from a dict."""
        raise NotImplementedError


class Source(metaclass=ABCMeta):
    """Abstract source interface, it enables retrieving data from it.

    Attributes:
        source_name: source identifier.
            Specific environment variables should be defined for each particular source.
        init_params: parameters used for source initialization.
    """

    @abstractmethod
    def __init__(self, source_name: str) -> None:
        self.source_name = source_name
        self.init_params = self._find_init_params_in_env_vars()

    def _find_init_params_in_env_vars(self) -> Dict[str, str]:
        """Try to find values in environment variables specified in params property.

        Returns:
            Dict of initialization parameters.
        """
        init_params = {}
        for param in self.params:
            env_name = param.env_name + "_" + self.source_name.upper()
            env_value = os.getenv(env_name, param.default)
            if env_value is not None:
                init_params[param.env_name] = env_value
            else:
                raise DatatoskEnvValueError(
                    f"Missing environment variable {env_name} value."
                )
        return init_params

    @property
    @abstractmethod
    def params(self) -> List[SourceParam]:
        """List of Source initialization parameters"""

    @property
    @abstractmethod
    def read(self) -> SourceRead:
        """Interface element for retrieving data from source."""

    @property
    @abstractmethod
    def write(self) -> SourceWrite:
        """Interface element for writing data to the source."""



from typing import List, Optional, Union

import pandas as pd  # type: ignore[import]
from pymongo import MongoClient  # type: ignore[import]
from pymongo.collection import Collection  # type: ignore[import]
from pymongo.cursor import Cursor  # type: ignore[import]
from pymongo.database import Database  # type: ignore[import]
from pymongo.results import InsertManyResult  # type: ignore[import]

from datatosk import types
from datatosk.sources.source import Source, SourceParam, SourceRead, SourceWrite


# pylint: disable = abstract-method
class MongoRead(SourceRead):
    """
    Interface class for reading data from MongoDB.

    Attributes:
        database: A MongoDB database name
    """

    def __init__(self, database: Database) -> None:
        self.database = database

    def __call__(
        self,
        collection: str = "",
        query_filter: types.DictType = None,
        projection: types.DictType = None,
    ) -> Cursor:
        collection_instance: Collection = self.database[collection]
        return collection_instance.find(filter=query_filter, projection=projection)

    def to_list(
        self,
        collection: str = "",
        query_filter: types.DictType = None,
        projection: types.DictType = None,
    ) -> types.ListType:
        collection_instance: Collection = self.database[collection]
        return list(
            collection_instance.find(filter=query_filter, projection=projection)
        )

    def to_pandas(
        self,
        collection: str = "",
        query_filter: types.DictType = None,
        projection: types.DictType = None,
    ) -> pd.DataFrame:
        return pd.DataFrame(
            self.to_list(
                collection=collection, query_filter=query_filter, projection=projection,
            ),
        )

    def to_dict(self) -> Union[types.DictType, types.JSONType]:
        raise NotImplementedError


class MongoWrite(SourceWrite):
    """
    Interface class for writing data to MongoDB.

    Attributes:
        database: A MongoDB database name
    """

    def __init__(self, database: Database) -> None:
        self.database = database

    def __call__(
        self,
        collection: str,
        data: Union[pd.DataFrame, types.DictType, types.ListType],
    ) -> InsertManyResult:
        collection = self.database[collection]
        if isinstance(data, pd.DataFrame):
            return self.from_pandas(df=data, collection=collection)

        raise NotImplementedError(f"Unsupported data type: {type(data)}")

    def from_pandas(
        self,
        df: Optional[pd.DataFrame] = None,
        collection: Optional[Collection] = None,
    ) -> InsertManyResult:
        if not isinstance(df, pd.DataFrame):
            raise ValueError("df argument must be a pandas dataframe")

        data = df.to_dict(orient="records")
        return self._execute_write(collection=collection, data=data)

    def from_dict(self, data: types.DictType) -> None:
        raise NotImplementedError

    def from_list(self, data: types.DictType) -> None:
        raise NotImplementedError

    @staticmethod
    def _execute_write(
        collection: Collection, data: Union[types.ListType, types.DictType],
    ) -> InsertManyResult:
        if isinstance(data, list):
            return collection.insert_many(documents=data)
        if isinstance(data, dict):
            return collection.insert_one(document=data)

        raise NotImplementedError("Unsupported data type")


class MongoSource(Source):
    """
    MongoDB database interface.

    Attributes:
        source_name: source identifier.
            Environment variables that should be defined:
            - `MONGO_HOST_[SOURCE_NAME]`
            - `MONGO_PORT_[SOURCE_NAME]`
            - `MONGO_DATABASE_[SOURCE_NAME]`
            - `MONGO_USER_[SOURCE_NAME]`
            - `MONGO_PASSWORD_[SOURCE_NAME]`

    Examples:

    Reading from MongoDB Source
    ===========================

    >>> import datatosk
    >>> mongo_source = datatosk.mongodb("source_name")
    >>> mongo_source.read.to_list(
        collection="vintage_guitars", command="find", query={"model": "stratocaster"},
    )
    [
      {
        '_id': ObjectId('5f4cc56f45f571f6532b57de'),
        'name': "Fender Standard Stratocaster Olympic White",
        'year_of_production': "1974",
        'price': 1340000
      }
    ]

    Writing to MongoDB Source
    ====================

    >>> import datatosk
    >>> mongo_source = datatosk.mongodb("source_name")
    >>> mongo_source.write(
        collection="vintage_guitars", command="insert_many", data=DataFrame()
    )

    The library will attepmt to automatically handle passed data type,
    but there's also a possibiliti to force a particular type:
    >>> mongo_source.write.from_dict(
        collection="vintage_guitars", command="insert_many", data={}
    )
    """

    def __init__(self, source_name: str) -> None:
        super().__init__(source_name)
        self.client = MongoClient(
            host=self.init_params["MONGO_HOST"],
            port=int(self.init_params["MONGO_PORT"]),
            username=self.init_params["MONGO_USER"],
            password=self.init_params["MONGO_PASSWORD"],
        )
        self.database = self.client[self.init_params["MONGO_DATABASE"]]

    @property
    def params(self) -> List[SourceParam]:
        return [
            SourceParam(env_name="MONGO_HOST"),
            SourceParam(env_name="MONGO_PORT"),
            SourceParam(env_name="MONGO_DATABASE"),
            SourceParam(env_name="MONGO_USER"),
            SourceParam(env_name="MONGO_PASSWORD"),
        ]

    @property
    def read(self) -> MongoRead:
        return MongoRead(database=self.database)

    @property
    def write(self) -> MongoWrite:
        return MongoWrite(database=self.database)



from typing import List

import pandas as pd  # type: ignore
import sqlalchemy

from datatosk import types
from datatosk.sources.source import Source, SourceParam, SourceWrite
from datatosk.sources.databases.sql.sql_source import SQLRead


class MySQLRead(SQLRead):
    """Interface element for retrieving data from MySQL database.

        Attributes:
            engine: SQLAlchemy engine
        """

    def __init__(self, engine: sqlalchemy.engine.Engine) -> None:
        self.engine = engine

    def to_pandas(
        self,
        query: str = "",
        params: types.ParamsType = None,
        **kwargs: types.KwargsType,
    ) -> pd.DataFrame:
        return pd.read_sql(query, con=self.engine, params=params, **kwargs)

    def to_list(
        self, query: str = "", params: types.ParamsType = None
    ) -> types.ListType:

        with self.engine.connect() as connection:

            if params is None:
                result = connection.execute(query)
            else:
                result = connection.execute(query, params)

            return [
                item[0] if len(item) == 1 else list(item) for item in result.fetchall()
            ]

    def to_dict(
        self, query: str = "", params: types.ParamsType = None,
    ) -> types.DictType:
        with self.engine.connect() as connection:

            if params is None:
                result = connection.execute(query)
            else:
                result = connection.execute(query, params)

            columns = [column[0] for column in result.keys()]

            return [dict(zip(columns, row)) for row in result.fetchall()]


class MySQLSource(Source):
    """MySQL database interface, it enables retrieving data from it.

    Attributes:
        source_name: source identifier.
            Envioronment variables that should be defined:
            - `MYSQL_USER_[SOURCE_NAME]`
            - `MYSQL_PASS_[SOURCE_NAME]`
            - `MYSQL_HOST_[SOURCE_NAME]`
            - `MYSQL_PORT_[SOURCE_NAME]`
            - `MYSQL_DB_[SOURCE_NAME]`
        engine: SQLAlchemy engine


    Examples:

    Reading MySQL Source
    ====================

    >>> import datatosk
    >>> source = datatosk.mysql(source_name="dope_db")
    >>> source.read("SELECT * FROM dope_table")
        dope_column  other_dope_column
    0             1                  3
    1             2                  4

    >>> source = datatosk.mysql(source_name="dope_db")
    >>> source.read.to_list("SELECT SUM(*) FROM dope_table")
    [3, 7]

    >>> source = datatosk.mysql(source_name="dope_db")
    >>> source.read.to_dict("SELECT * FROM dope_table")
    [{dope_column: 1, other_dope_column:3}],
    [{dope_column: 2, other_dope_column:4}]

    Use of params
    -------------

    >>> source = datatosk.mysql(source_name="dope_db")
    >>> source.read(
    ...     "SELECT dope_column, other_dope_column"
    ...     "FROM dope_table"
    ...     "WHERE dope_column > %(dope_col)s",
    ...     params={"dope_col": 1}
    ... )
        dope_column  other_dope_column
    0             2                  4
    """

    def __init__(self, source_name: str) -> None:
        super().__init__(source_name)

        self.engine = sqlalchemy.create_engine(
            f"mysql+mysqldb://{self.init_params['MYSQL_USER']}:"
            f"{self.init_params['MYSQL_PASS']}@"
            f"{self.init_params['MYSQL_HOST']}:"
            f"{self.init_params['MYSQL_PORT']}/"
            f"{self.init_params['MYSQL_DB']}"
            "?charset=utf8mb4",
            pool_recycle=3600,
        )

    @property
    def params(self) -> List[SourceParam]:
        return [
            SourceParam(env_name="MYSQL_USER"),
            SourceParam(env_name="MYSQL_DB"),
            SourceParam(env_name="MYSQL_PASS", is_required=False, default="",),
            SourceParam(env_name="MYSQL_HOST", is_required=False, default="localhost",),
            SourceParam(env_name="MYSQL_PORT", is_required=False, default="3306",),
        ]

    @property
    def read(self) -> MySQLRead:
        """Interface element for retrieving data from MySQL database.

        Returns:
            Method-like class which can be used to retrieve data in various types.
        """
        return MySQLRead(self.engine)

    @property
    def write(self) -> SourceWrite:
        """Not implemented yet"""
        raise NotImplementedError



from importlib.metadata import version, PackageNotFoundError

from . import sources, types
from .main import gbq, mysql, pickle, requests, mongodb

__all__ = ["gbq", "mysql", "requests", "mongodb", "pickle", "types", "sources"]

try:
    __version__ = version(__name__)
except PackageNotFoundError:  # pragma: no cover
    __version__ = "unknown"



import logging

from typing import List

import requests

from datatosk import types
from datatosk.errors import HttpRequestFailedError
from datatosk.sources.source import Source, SourceParam, SourceRead, SourceWrite


# pylint: disable = abstract-method
class RequestsRead(SourceRead):
    """Interface element for retrieving data through get requests.

    Attributes:
        url: End-point URL configuration.
    """

    def __init__(self, url: str) -> None:
        self.url = url

    def __call__(self, path: str = "", **kwargs: types.KwargsType,) -> types.JSONType:
        """Default RequestsSource().read() implementation."""
        return self.to_dict(path=path, **kwargs)

    def to_dict(self, path: str = "", **kwargs: types.KwargsType,) -> types.JSONType:
        full_url = self.url + path
        logging.info("Getting response from request to %s", full_url)
        response = requests.get(full_url, **kwargs)

        if not response.ok:
            raise HttpRequestFailedError(
                f"Server responded with status code {response.status_code}"
            )

        json_response: types.JSONType = response.json()
        return json_response


class RequestsSource(Source):
    """Requests interface, it enables retrieving data through get request.

    Attributes:
        source_name: source identifier.
            Envioronment variables that should be defined:
            - `HTTP_PROTOCOL_[SOURCE_NAME]`
            - `HTTP_HOST_[SOURCE_NAME]`
            - `HTTP_PORT_[SOURCE_NAME]`
        url: End-point URL configuration.

    Examples:
        >>> import datatosk
        >>> source = datatosk.requests(source_name="chuck_norris_facts")
        >>> source.read(path="jokes/random/")
    """

    def __init__(self, source_name: str) -> None:
        super().__init__(source_name)

        self.url = (
            f"{self.init_params['HTTP_PROTOCOL']}://"
            f"{self.init_params['HTTP_HOST']}:"
            f"{self.init_params['HTTP_PORT']}/"
        )

    @property
    def params(self) -> List[SourceParam]:
        return [
            SourceParam(env_name="HTTP_PROTOCOL", is_required=False, default="http"),
            SourceParam(env_name="HTTP_HOST", is_required=False, default="localhost"),
            SourceParam(env_name="HTTP_PORT", is_required=False, default="80"),
        ]

    @property
    def read(self) -> RequestsRead:
        """Interface element for retrieving data from get requests.

        Returns:
            Method-like class which can be used to retrieve data in various types.
        """
        return RequestsRead(self.url)

    @property
    def write(self) -> SourceWrite:
        """Not implemented yet"""
        raise NotImplementedError



import pickle
from typing import Any, Dict, List

from datatosk.sources.source import Source, SourceParam, SourceRead, SourceWrite


# pylint: disable = abstract-method
class PickleRead(SourceRead):
    """Interface element for retrieving data from pickle files.

    Attributes:
        filepath: pickle file path
    """

    def __init__(self, filepath: str) -> None:
        self.filepath = filepath

        with open(self.filepath, "rb") as file:
            self.obj = pickle.load(file)

    def __call__(self) -> object:
        """Default PickleSource().read() implementation."""
        return self.obj

    def to_dict(self) -> Dict[Any, Any]:
        if isinstance(self.obj, dict):
            return self.obj
        try:
            return dict(self.obj)
        except (ValueError, TypeError) as err:
            raise ValueError("This object cannot be changed to dict.") from err


class PickleSource(Source):
    """Pickle files interface, it enables retrieving data from it.

    Attributes:
        source_name: source identifier.
            Envioronment variables that should be defined:
            - `PICKLE_FILEPATH_[SOURCE_NAME]`
        filepath: pickle file path

    Examples:
        >>> import datatosk
        >>> source = datatosk.pickle(filepath="cool_file.pickle")
        >>> source.read()
        <cool object>

        >>> import datatosk
        >>> source = datatosk.pickle(filepath="cool_file.pickle")
        >>> source.read.to_dict()
        {"pickle": "tasty"}
    """

    def __init__(self, source_name: str) -> None:
        super().__init__(source_name)

        self.filepath = self.init_params["PICKLE_FILEPATH"]

    @property
    def params(self) -> List[SourceParam]:
        return [SourceParam(env_name="PICKLE_FILEPATH")]

    @property
    def read(self) -> PickleRead:
        """Interface element for retrieving data from pickle file.

        Returns:
            Method-like class which can be used to retrieve data in various types.
        """
        return PickleRead(self.filepath)

    @property
    def write(self) -> SourceWrite:
        """Not implemented yet"""
        raise NotImplementedError



from typing import List, Union, Final
from datetime import datetime, date

from google.cloud.bigquery import ScalarQueryParameter, ArrayQueryParameter  # type: ignore[import]
import pandas as pd  # type: ignore[import]
import pandas_gbq  # type: ignore[import]

from datatosk import types
from datatosk.sources.databases.sql.sql_source import SQLRead
from datatosk.sources.source import Source, SourceParam, SourceWrite


class BigqueryTypes:  # pylint: disable=too-few-public-methods
    """GBQ data types."""

    BOOL: Final[str] = "BOOL"
    INT64: Final[str] = "INT64"
    FLOAT64: Final[str] = "FLOAT64"
    DATE: Final[str] = "DATE"
    TIMESTAMP: Final[str] = "TIMESTAMP"
    STRING: Final[str] = "STRING"


class GBQRead(SQLRead):
    """Interface element for retrieving data from GBQ dataset.

    Attributes:
        project_id: GoogleBigQuery project_id
    """

    def __init__(self, project_id: str) -> None:
        self.project_id = project_id

    def to_list(self) -> types.ListType:
        raise NotImplementedError

    def to_dict(self) -> types.DictType:
        raise NotImplementedError

    def to_pandas(
        self,
        query: str = "",
        params: types.ParamsType = None,
        **kwargs: types.KwargsType,
    ) -> pd.DataFrame:

        return pandas_gbq.read_gbq(
            query=query,
            project_id=self.project_id,
            configuration={
                "query": {"queryParameters": self._cast_params(params=params)}
            },
            **kwargs,
        )

    def _cast_params(self, params: types.ParamsType = None) -> List[types.JSONType]:

        if params is None:
            return []

        for param_name, param_value in params.items():

            if isinstance(param_value, (list, tuple)):

                assert all(
                    isinstance(item, type(param_value[0])) for item in param_value
                ), f"Items in `{param_name}` are inconsistent"

                param_val = param_value[0]
                param_type = ArrayQueryParameter
            else:
                param_val = param_value
                param_type = ScalarQueryParameter

            param_type(
                param_name, self._recognize_type(param_val), param_value
            ).to_api_repr()

        return []

    @staticmethod
    def _recognize_type(  # pylint: disable=too-many-return-statements
        param_value: Union[bool, float, int, datetime, date, str]
    ) -> str:

        if isinstance(param_value, bool):
            return BigqueryTypes.BOOL

        if isinstance(param_value, int):
            return BigqueryTypes.INT64

        if isinstance(param_value, float):
            return BigqueryTypes.FLOAT64

        if isinstance(param_value, datetime):
            return BigqueryTypes.TIMESTAMP

        if isinstance(param_value, date):
            return BigqueryTypes.DATE

        if isinstance(param_value, str):
            try:
                datetime.fromisoformat(param_value)
            except ValueError:
                return BigqueryTypes.STRING
            else:
                return BigqueryTypes.TIMESTAMP

        raise ValueError(f"Parameter type {type(param_value)} is not supported.")


class GBQSource(Source):
    """GBQ database interface, it enables retrieving data from it.

    Attributes:
        source_name: source identifier.
            Envioronment variables that should be defined:
            - `GBQ_PROJECT_ID_[SOURCE_NAME]`
        project_id: GoogleBigQuery project_id

    Examples:

    Reading GoogleBigQuery Source
    ====================

    >>> import datatosk
    >>> source = datatosk.gbq(source_name="epic_source")
    >>> source.read("SELECT * FROM epic_table")
        superheros        real_name
    0      Ironman       Tony Stark
    1       Batman      Bruce Wayne
    2     Catwoman      Selina Kyle

    Use of params
    -------------

    >>> source = datatosk.gbq(source_name="epic_source")
    >>> source.read(
    ...     "SELECT superheros, real_name"
    ...     "FROM epic_table"
    ...     "WHERE superheroes = @superhero",
    ...     params={"superhero": "Catwoman"}
    ...)
        superheros        real_name
    0     Catwoman      Selina Kyle
    """

    def __init__(self, source_name: str) -> None:
        super().__init__(source_name)
        self.project_id = self.init_params["GBQ_PROJECT_ID"]

    @property
    def params(self) -> List[SourceParam]:
        return [SourceParam(env_name="GBQ_PROJECT_ID")]

    @property
    def read(self) -> GBQRead:
        """Interface element for retrieving data from GBQ dataset.

        Returns:
            Method-like class which can be used to retrieve data in various types.
        """
        return GBQRead(self.project_id)

    @property
    def write(self) -> SourceWrite:
        """Not implemented yet"""
        raise NotImplementedError



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['datatosk',
 'datatosk.consts',
 'datatosk.sources',
 'datatosk.sources.databases',
 'datatosk.sources.databases.sql']

package_data = \
{'': ['*']}

install_requires = \
['google-cloud-bigquery>=1.24.0,<2.0.0',
 'mysqlclient>=2.0.1,<3.0.0',
 'pandas-gbq>=0.13.1,<0.14.0',
 'pandas>=1.0.1,<2.0.0',
 'pymongo>=3.11.0,<4.0.0',
 'sqlalchemy>=1.3.13,<2.0.0']

setup_kwargs = {
    'name': 'datatosk',
    'version': '1.1.0a1',
    'description': 'Python library for fetching data from different sources',
    'long_description': '<div style="text-align: center"> \n<img src="datatosk_logo.jpg" alt="Datatosk">\n<h1>Datatosk</h1>\n</div>\n\n> Python library for fetching data from different sources\n\n# Usage\nDatatosk reads configuration from the `environment variables`.\n\n## MySQL\n\nProvide particular enviroment variables:\n```\nMYSQL_HOST_[SOURCE_NAME]=\nMYSQL_PORT_[SOURCE_NAME]=\nMYSQL_USER_[SOURCE_NAME]=\nMYSQL_PASS_[SOURCE_NAME]=\nMYSQL__DB_[SOURCE_NAME]=\n```\n\n## GoogleBigQuery\n\nProvide particular enviroment variables:\n```\nGBQ_PROJECT_ID_[SOURCE_NAME]=\n```\n',
    'author': 'Miłosz Bednarzak',
    'author_email': 'milosz.bednarzak@bethink.pl',
    'maintainer': 'Miłosz Bednarzak',
    'maintainer_email': 'milosz.bednarzak@bethink.pl',
    'url': 'https://github.com/bethinkpl/datatosk',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
